import { Body, Controller, Get, Inject, Param, Post, Req, Res } from '@nestjs/common';
import { ClientProxy } from '@nestjs/microservices';

// import { RequestDto } from './request-dto';

import { from, map, observable, Observable, of, Subscription } from 'rxjs';
import {
  FisQueryMessage,
  FisUASRequestMessage,
  ResponseMessage,
} from './module/message/fis.request.message';

/**
 * Client query handler
 *
 * @class AppController
 */

@Controller('/')
export class AppController {
  constructor(@Inject('FFB_SERVICE') private client: ClientProxy) {
    
    this.requestCmd('QUERY',{"payload":this.payload}).subscribe(result=>{
      console.log(result)
    })

  }

  /**
   * client GET request.
   * @params / {database} / {column} / phases(optional) / phases column(optional) / blocks(optional) / block column(optional)
   * @returns {FisAppRequestMessage} - Fis App request message.
   */

  public MessageService: FisUASRequestMessage;

  result: any;

  header = ` {
      "messageType": "Query",
      "messageID": "942ecfe2-8734-4cff-998e-6a2a5a39c464",
      "messageName": "Query",
      "dateCreated": "2022-03-11T02:19:50.923Z",
      "isAggregate": false,
      "messageProducerInformation": {
        "origin": {
          "userApplication": {
            "userAppId": "getQueryMessage",
            "userAppName": "Client"
          }
        },
        "components": "Presentation"
      },
      "security": {
        "ucpId": "ABC"
      },
      "messageDataLocation": {
        "isEmbaded": true
      },
      "messageDataFormat": {
        "dataFormat": "Json"
      },
      "requestExecutionMode": 0,
      "resquestTimeOut": 0,
      "query": "General"
    }
  `;

  payload =  `query demo{
        APAccountProfileQuery{
          SummaryResponse{
            __typename
            returnFormatType
          }
          StatusResponse{
            __typename
            status
          }
          APAccountProfileMessage{
            __typename
            data{
              acct_id
              acct_no
            }
          }
        }
      }`;
  
  requestCmd(cmd, params = {}) {

    if (!this.MessageService) {
      this.MessageService = new FisUASRequestMessage('getQueryMessage');
    }


    return this.requestCmdRequest(cmd, this.MessageService.getQueryMessage("ABC", params));
  }


  
  requestCmdRequest(cmd, params: FisQueryMessage) {

    const pattern = { cmd: cmd };
    let storearray = Array();

    if (!this.MessageService) {
      this.MessageService = new FisUASRequestMessage('getQueryMessage');
    }

    // let newPromise = new Promise((resolve, reject) => {
      let newObservable = new Observable((subscriber)=>{
        this.client
          .send<string, FisQueryMessage>(
            pattern,
             params,
          )
          .subscribe({
            next: (x) => {
              // storearray.push(JSON.stringify(x, null, 10));
              // console.log(x)
              subscriber.next(x)
            },
            error: (err) => {
              // reject(err)
              subscriber.error(err)
            },
            complete: () => {
              subscriber.complete()
              // resolve(storearray.join(','));
            },
      })

     

        // });
    });

    
    return newObservable;    
    // return newPromise;
  }
}
