/* eslint-disable @typescript-eslint/no-unused-vars */
/* eslint-disable prefer-const */
import {
  Command,
  DataFormat,
  FisAppRequestMessage,
  MessageParameter,
  ProducerType,
  Subscription,
} from './fisappmessagejslib/fis/fismessagetype';
import { AppMessageCreator } from './fisappmessagejslib/message/appmessagecreator';
import {
  FisQueryMessage,
  AppMessageType,
  CommandMessageParameter,
  QueryMessageParameter,
  SubscriptionMessageParameter,
  ResponseMessage,
  Query,
} from './fisappmessagejslib/message/request/apprequestmessagetype';
import {
  FisCommandMessageParameter,
  FisQueryMessageParameter,
  FisResponseMessage,
} from './fisappmessagejslib/fis/fisrequestmessagetype';
//import { v4 as uuidv4 } from 'uuid';
import { AppResponseMessage } from './fisappmessagejslib/message/response/appresponsemessage';

// Use this exported type. Note:The typing might be changed due to UCP server.
export {
  FisQueryMessage,
  ResponseMessage,
} from './fisappmessagejslib/message/request/apprequestmessagetype';

export class FisUASRequestMessage {
  private thisUserAppId = ''; // Required
  private PRODUCER = {
    type: ProducerType.UI,
    origin: {
      userApplication: {
        userAppId: this.thisUserAppId,
        userAppName: 'Client',
      },
    },
  };

  constructor(UserAppId: string) {
    // Initial setting
    this.setUserAppId(UserAppId); // Must set to something meaningful
  }

  private setUserAppId(UserAppId: string) {
    this.thisUserAppId = UserAppId;
    this.PRODUCER.origin.userApplication.userAppId = UserAppId;
  }

  public getUserAppId() {
    return this.thisUserAppId;
  }

  public getQueryMessage(
    ucpId: string,
    data,
    messageName?: string,
  ): FisQueryMessage {
    let messageParm: QueryMessageParameter = {
      security: { ucpId: ucpId },
      producer: this.PRODUCER,
      messageType: AppMessageType.Query,
      messageName: messageName ? messageName : 'Query',
      data: data,
      isAggregate: false,
      dataLocation: { isEmbaded: true },
      dataFormat: { dataFormat: DataFormat.Json },
      query: Query.General,
    } as QueryMessageParameter;

    return AppMessageCreator.create(messageParm) as FisQueryMessage;
  }

  public getResponseMessage(
    ucpId: string,
    data: string,
    messageName?: string,
  ): ResponseMessage {
    let AppResponseCreator = new AppResponseMessage();
    let messageParm: MessageParameter = {
      security: { ucpId: ucpId },
      producer: this.PRODUCER,
      messageType: AppMessageType.Response,
      messageName: messageName ? messageName : 'Response',
      data: data,
      isAggregate: false,
      dataLocation: { isEmbaded: true },
      dataFormat: { dataFormat: DataFormat.Json },
      query: Query.General,
    } as MessageParameter;

    return AppResponseCreator.create(messageParm) as ResponseMessage;
  }
}
